/* Example of programming the serial port using interrupts

Inside the interrupt if bit 6 of SP_CON is set it's at RX interrupt, otherwise
it's a TX interrupt.  You can only read SP_CON once because the RI and TI flags
are cleared after the first read.  That data on RX interrupts is available in
the SBUF register.

Bits in the SP_CON register

	0, 1:		Mode of operation
					
					[0, 0]	Mode 0, Normal 8 bit shift mode
					[0, 1]  Mode 1,	10 bit mode (start and stop bits)
					[1, 0]	Mode 2, 11 bit mode (start, stop and programmable bit)
					[1, 1]	Mode 3, Similar to mode 2

	2:			Enable even parity
	3:			Receieve enable
	4:			9th data bit
	5:			TI flag (transmit interrupt)
	6:			RI flag (receive interrupt)
	7:			9th bit received (if bit 2 = 0)
				Parity error (if bit 2 = 1)

*/

#include <8096io.h>
#include <8096int.h>

asm {
	#include "regs.asm"
}

#include "8096reg.h"

/* serial interrupt */
INTERRUPT(8204) ser_int() {

	/* Check which interrupt occured */
	if (SP_CON & 0x40) {
		/* receive interrupt */
		SBUF = SBUF + 1;						/* re-transmit the data */
	} else {
		/* transmit interrupt */
	}
}

main()
{
	/* serial port inited, just set int_mask */
	SP_STAT   = 0x69; 					/* Init the serial port */
	INT_MASK  = 0x40;					/* Enable serial interrupts */
	enable();

	for (;;);							/* loop forever */
}
