/* 
	Count down example using the Timer1, which has a interrupt vector
at 8192.

	The timer is set to trigger a overflow interrupt once every 100ms.  Since
the timer is at a fixed rate we only have to enable the timer.  Serial 
interrupts are left enabled (you can only write to the control register
so we have to make assumptions).
*/

#include <8096io.h>
#include <8096int.h>

/* register defines */
asm {
	#include "regs.asm"
}

#include "8096reg.h"

#define TIME		100				/* 10 ticks per second at 12 Mhz */

unsigned irq_count;

/* Timer Overflow handler */
INTERRUPT(8192) irq_timer()
{
	--irq_count;					/* decrease count */
}

main()
{
	printf("Waiting...\n\r");

	irq_count = TIME;				/* Count down */
	IOC1 = 0x24;					/* enable timer */
	INT_MASK = 0x41;				/* enable OVF interrupt */
	enable();

	while (irq_count)
		printf("%5u\r", irq_count);

	printf("\nGood.\n\r");

	for (;;);
}
