/* Example of reading and writing values into the AVR-EEPROM

	For activating the write on EEPROM you must perform it in assembler
as the master enable and write enable must be within 4 clock cycles.

	It is also required to disable interrupts around writes to eeprom as it 
may interrupt the write cycle.  Ideally you would write in blocks and only
have to disable interrupts for a short manner of time.
*/

#include <avrio.h>
#include <avrreg.h>
#include <avrint.h>

/* write one byte at the specified address in EEPROM */
write_eeprom(unsigned address, unsigned char value)
{
	while(EECR & 2);			/* wait for ok */
	EEAR  = address;			/* write address */
	EEDR  = value;				/* value to write */
	asm {
			SBI $1C,2			/* set master write */
			NOP					/* waste a couple of cycles */
			NOP	
			SBI $1C,1			/* perform write */
	};
}

/* read one byte from EEPROM */
unsigned char read_eeprom(unsigned address)
{
	while(EECR & 2);			/* make sure no write is being performed */
	EEAR  = address;			/* read address */
	EECR |= 1;					/* toggle read operation */
	return EEDR;				/* return EEPROM value */
}
	
main()
{
	unsigned i;
	
	disable();					/* disable interrupts */
	for(i = 0; i < 256; i++)
		write_eeprom(i, i);		/* write values to eeprom */
	enable();					/* enaable interrupts */

	for(i = 0; i < 256; i++)
		printf("%u\n", read_eeprom(i));

	for(;;) {}					/* endless loop */
}
