/* Example using Timer1 with all three modes of operation.

	In this example we set Timer1 to CK/256, then cause interrupts 
when the timer count equals 0xAAAA, 0x5555 and on overflows.  The AVR
has two compare registers for timer1, OCR1A and OCR1B.  If those compares
are enabled (bit 6 and 5 of TIMSK respectively) then an interrupt will
occur when the timer equals them.  Also if bit 7 of TIMSK is enabled interrupts
will occur when the 16-bit Timer1 counter overflows.  OCR1A and OCR1B are
both 16-bit values which are used to compare against the Timer1 count.

	CK = input clock frequency in hz.
*/

#include <avrio.h>
#include <avrint.h>
#include <avrreg.h>

#include "def_avr.h"

unsigned char counter;

/* Note OCR1A and OCR1B must be written in MSB first format.  Micro-C follows
the convention of LSB then MSB which will not set the I/O registers correctly. */
void write_reverse(unsigned char *address, unsigned value)
{
	address[1] = value >> 8;			/* MSB */
	*address = value;					/* LSB */
}

main()
{

	DDRB = 0xFF;                /* Set I/O direction */
	counter = 0xFF;             /* clear leds */
	TCCR1A  = 0x00;             /* Disconnect pin OCA1 and OCB1 */
	TCCR1B  = CK_256;           /* CK / 256 on timer1 */
	TIMSK   |= TOIE1 | OCE1A |  OCE1B; /* Enable ints on timer1 (bits 5,6,7) */

	write_reverse(&OCR1A, 0xAAAA);	/* Set the value of OCR1A */
	write_reverse(&OCR1B, 0x5555);	/* Set the value of OCR1B */
	
	enable();   				/* Enable interrupts */

	for (;;)
		PORTB = counter;		/* loop endlessly and set the LEDs */
}

/* interrupt on overflow */
INTERRUPT(_TIMER1OVF_) ovf_handler()
{
	counter ^= 0x04;			/* toggle LED #3 */
}

/* interupt on COMPAREA of timer 1 */
INTERRUPT(_TIMER1COMPA_) cmpa_handler()
{
	counter ^= 0x02;			/* toggle LED #2 */
}

/* interrupt on COMPAREB of timer 1 */
INTERRUPT(_TIMER1COMPB_) cmpb_handler2()
{
	counter ^= 0x01;			/* Toggle LED #1 */
}
