/* Example of using serial interrupts with a AVR.

In our example PORTB is hooked upto 8 LED's.  Whenever a byte is received
a 4-bit counter will be updated, then the character will be sent back through
the UART causing a TX interrupt.  The TX int will update another 4-bit counter.
*/

#include <avrio.h>
#include <avrint.h>
#include <avrreg.h>

#include "def_avr.h"

unsigned char countera, counterb;

main()
{
	countera = counterb = 0;	/* clear counts */

	DDRB 	= 0xFF;				/* Enable I/O */
	UCR 	|= RXCIE | TXCIE; 	/* enable interrupts on RX and TX */
	enable();					/* enable interrupts */

	for (;;)
		PORTB = (countera << 4) | counterb;	/* loop endlessly */
}

INTERRUPT(_UARTRX_) rx_int()
{
	countera = (countera + 1) & 15;	/* update counter */
	UDR = UDR;			/* send the data back thru the port */
}

INTERRUPT(_UARTTX_) tx_int()
{
	counterb = (counterb + 1) & 15; /* update counter */
}
