/* Stopwatch AVR demo. Uses one timer interrupt, and two external interrupts.

	When you push PB #3 it will start the timer.  And if you press PB #4 it
will stop the timer.  The main() function will loop displaying the number
of ticks as long as the timer is running.

	Timer0 is set to interrupt on counter overflow, with a prescaler of
CK/1024.

	IDLE mode is toggled after every tick.

	You cannot use printf inside an interrupt.  However you can use putch().
Also it is a good idea to disable interrupts during printf() calls as you may
run out of stack space.
*/

#include <avrio.h>
#include <avrreg.h>
#include <avrint.h>

#include "def_avr.h"			/* Defines */

/* number of ticks and current state */
unsigned start, active;

/* place the cpu in sleep mode.  The only thing that can wake it is an
interrupt */
void sleep(int acd, int mode)
{
	/* turn ACD off? */
	if(acd)
		ACSR |= 0x80;				/* disable ACD */
	else
		ACSR &= 0x7F;				/* leave it enabled */

	/* Idle or Powerdown mode? */
	if(mode)
		MCUCR &= ~SM;				/* Idle mode */
	else
		MCUCR |= SM;				/* Power down */

	MCUCR	  |= SE;				/* enable SLEEP command */
	asm "SLEEP";					/* Enter sleep mode */
}

main()
{
	active = start = 0;		/* clear count */

	DDRB	= 0xFF;			/* enable leds */
	PORTB	= 0xFF;			/* clear leds */

	TCCR0 	= CK_1024;		/* CK/1024 */
	TIMSK  |= TOIE0;		/* enable overflow on Timer 0 */
	GIMSK   = INT1 | INT0;	/* enable both external interrupts */
	UCR    &= 0x0F;			/* disable ints on TX and RX, and disable RX */
	enable();				/* enable global interrupts */

	printf("\nTimer demo\n");					/* greeting */
	for(;;) { 									/* endless loop */
		if(active && !(start & 15)) {			/* every 16 ticks */
			disable();							/* disable interrupts */
			printf("Ticks so far: %5u\r", start);
			enable(); }							/* enable interrupts */
		sleep(1, 1); }							/* enter sleep mode */
}

/* timer0 overflow */
INTERRUPT(_TIMER0OVF_) timer_handler()
{
	if(active) {
		++start;					/* update count */
		++PORTB; }					/* update LEDs */
}

/* Push button #3 or external interrupt #0 */
INTERRUPT(_INT0_) int0_handler()
{
	active = 1;						/* activate timer */
}

/* Push button #4 or external interrupt #1 */
INTERRUPT(_INT1_) int1_handler()
{
	active = start = 0;			/* disable timer */
}
