/* Example of programming the Watchdog Timer 

	In this example the timer will be set to 2048ms, and reset every
2 seconds.  You have to disable the WDT in asm as you have only four cycles
to disable the WDE bit.
*/

#include <avrio.h>
#include <avrint.h>
#include <avrreg.h>

#include "def_avr.h"

/* enable the WDT at the speed specified (see def_avr.h for the speeds) */
enable_wdt(unsigned int delay)
{
	WDTCR |= WDE | delay;	/* enable WDT and delay */
}

/* Turn off the WDT */
disable_wdt() asm {
	IN R16,$21              /* get WDTCR */
	ORI R16,24              /* enable both (WDE and WDTTOE) */
	OUT $21,R16
	ANDI R16,$17			/* disable WDE */
	NOP						/* waste 2 cycles */
	NOP
	OUT $21,R16				/* set WDTCR */
}


main()
{
	printf("Will reset in 2048ms\n\n");
	enable_wdt(WD_2048);			/* enable WDT at 2048ms */

	getch();                        /* get a char */
	disable_wdt();                  /* turn off WDT */
	printf("WDT stopped.\n\n");
	for(;;) {}						/* loop */
}
