/*
	In this example the RTI and TOF interrupts are enabled.  However
you must set the prescale for both timers within the first 64 cycles.  If
you are running straight C code this is fine, but under a monitor this
can cause problems.

	You must place your vector address inside the INTERRUPT macro.  For testing
I used the MON11 package, and manually updated the vector (CV command).

	Compile with:  cc11 tim -pof
*/

#include <6811reg.h>
#include <6811io.h>
#include <6811int.h>

#define _RTI_	65520		/* RTI interrupt vector (FFF0) */
#define	_TOF_	65502		/* TOF interrupt vector (FFDE) */

/* counters */
int rti_count, toi_count;

/* Timer Overflow Interrupt */
INTERRUPT(_TOF_) toi_int()
{
	TFLG2 |= 0x80;				/* Clear TOF */
	++toi_count;
}

/* Real Time Interrupt */
INTERRUPT(_RTI_) rti_int()
{
	TFLG2 |= 0x40;				/* Clear RTIF */
	--rti_count;
}

main()
{
	rti_count = toi_count = 0;

	/* setup timers */
	TMSK2  = 0xE3;					/* RTI and TOI, and /16 prescale */
	PACTL  = 0x03;					/* set RTI pre-scale (/16) */
	enable();

	/* loop forever */
	for(;;)
		printf("%5u %5u\r", toi_count, rti_count);
}
