/* 
	Example of using real time interrupt, which is set to 131ms or 65ms
depending on the clock (4mhz or 8mhz).  In the RTI you must set the RTIF
flag in the RTIFLG register for the interrupt to clear.  The real time 
interrupt provides a simple and easy to program regular interrupt.

	On the DBG12 EVB you have to use different code to use interrupts.  
Basically it involves patching the interrupt.
*/

#include <6812io.h>
#include <6812int.h>
#include <6812reg.h>

int irq_count;

#define _SP_
#define DBG12

#define SetVect(vector, routine)	\
asm { 						\
	ldd _SP_ #routine		\
	pshd					\
	ldd _SP_ #vector		\
	JSR	_SP_ [$FE1A-*-4,PC]	\
	pulx					\
}

/* interrupt vector */
#ifndef DBG12
	INTERRUPT(0xFFF0)
#else
	asm {
dbg_jmp
	BSR rti_handler
	RTI
}
#endif

rti_handler()
{
	RTIFLG |= 0x80;						/* must set RTI flag in RTIFLG register */
	++irq_count;	
}

main()
{
	irq_count = 0;						/* clear count */
	
	/* Set interrupt vector (DBUG12) */
#ifdef DBG12
	SetVect(24, dbg_jmp);
#endif

	/* setup Timer */
	RTICTL = 0x87;						/* enable RTI and slow count */
	enable();							/* enable interrupts */

	/* loop */
	for(;;)							/* loop forever */
		printf("%5u\r", irq_count);
}