/* 
	In this example we set the Timer to overflow at a rate of CK/32.  No
other timer related interrupts or conditions are set.

	Under the DBG12 the actual placement of the interrupt is dynamic, so we
patch it using a simple jump. The benefit is that we can still write the
interrupt handler in c.
*/

#include <6812io.h>
#include <6812int.h>
#include <6812reg.h>

/* Number of irqs that have occured */
int irq_count;

/* used for a space */
#define _SP_

#define DBG12					/* debug12 monitor only */

#ifdef DBG12
/* macro to set vectors in DBG12 */
#define SetVect(vector, routine)	\
asm { 						\
	ldd _SP_ #routine		\
	pshd					\
	ldd _SP_ #vector		\
	JSR	_SP_ [$FE1A-*-4,PC]	\
	pulx					\
}
#endif

/* interrupt vector */
#ifndef DBG12
	INTERRUPT(0xFFDE) 
#else
	asm {
dbg_jmp
	BSR handler							/* your handler */
	RTI
}
#endif

handler()								/* irq handler */
{
	TCNT;								/* read counter */
	++irq_count;
}

main()
{
	irq_count = 0;						/* clear count */
	
	/* Set interrupt vector (DBUG12 only) */
#ifdef DBG12
	SetVect(23, dbg_jmp);				/* install TimerOvf handler */
#endif

	/* setup Timer */
	TSCR  = 0x90;						/* enable timer (and TOF reset) */
	TMSK2 = 0x85;						/* enable overflow (CK/32) */
	enable();							/* enable interrupts */

	/* loop */
	for(;;)
		printf("%5u\r", irq_count);
}