/* 
	This is an example of using the timer compare registers.  When the
free-running timer counter equals the value in one of the 7 timer compare
registers, an interrupt will occur. 

	In this example timer-compare0 is set to 0x8000 and the overflow is
enabled.  

	Also used is a macro so that you can use the INTERRUPT vector 
under DBUG12.
*/

#include <6812io.h>
#include <6812int.h>
#include <6812reg.h>

/* Number of irqs that have occured (and compare irqs) */
int irq_count, com_count;

/* used for a space */
#define _SP_

#define DBG12                   /* debug12 monitor only */

#define SetVect(vector, routine)    \
asm {                       \
	ldd _SP_ #routine       \
	pshd                    \
	ldd _SP_ #vector        \
	JSR _SP_ [$FE1A-*-4,PC] \
	pulx                    \
}

/* interrupt vector */
#ifndef DBG12
	INTERRUPT(0xFFDE) 
#else
	asm {
dbg_jmp1
	BSR ovf_handler						/* your handler */
	RTI
}
#endif
ovf_handler()                           /* overflow handler */
{
	TCNT;                               /* read counter */
	++irq_count;
}

#ifndef DBG12
	INTERRUPT(0xFFEE)
#else
	asm {
dbg_jmp2
	BSR com_handler
	RTI
}
#endif

com_handler()							/* compare handler */
{
	TC0 = TC0;							/* must write to the compare */
	++com_count;
}	

main()
{
	com_count = irq_count = 0;			/* clear count */
	
	/* Set interrupt vector (DBUG12 only) */
#ifdef DBG12
	SetVect(23, dbg_jmp1);                   /* install TimerOvf handler */
	SetVect(13, dbg_jmp2);					 /* Install compare */
#endif

	/* Setup Timer */
	TSCR    = 0x90;                     /* enable timer (and TOF reset) */
	TIOS    = 1;                        /* enable output compare on 1 */
	TC0     = 0x8000;                   /* compare channel 1 */
	TMSK2   = 0x85;                     /* enable overflow (CK/32) */
	TMSK1   = 0x01;                     /* Enable compare 1 */
	enable();

	for(;;)
		printf("%5u %5u\r", irq_count, com_count);
}